@@ -13,6 +13,7 @@
     
 #include <vdr/interface.h>
 #include <vdr/i18n.h>
+#include <vdr/config.h>
 
 #include "menu.h"
 #include "display.h"
@@ -60,6 +61,8 @@ TeletextBrowser::TeletextBrowser(cTxtStatus *txtSt) {
    previousPage=currentPage;
    previousSubPage=currentSubPage;
    pageBeforeNumberInput=currentPage;
+   lastActivity=time(NULL);
+   inactivityTimeout=-1;
 }
 
 
@@ -97,7 +100,6 @@ void TeletextBrowser::ChannelSwitched(int ChannelNumber) {
    
    //store page number of current channel
    IntMap::iterator it;
-   //channelPageMap.insert(IntMap::value_type(currentChannelNumber, currentPage));
    channelPageMap[currentChannelNumber] = currentPage;
    currentChannelNumber=ChannelNumber;
    
@@ -119,7 +121,10 @@ void TeletextBrowser::ChannelSwitched(int ChannelNumber) {
 }
 
 
-eOSState TeletextBrowser::ProcessKey(eKeys Key) {  
+eOSState TeletextBrowser::ProcessKey(eKeys Key) {
+   if (Key != kNone)
+      lastActivity = time(NULL);
+   
    switch (Key) {
       case k1: SetNumber(1);break;
       case k2: SetNumber(2);break;
@@ -166,6 +171,9 @@ eOSState TeletextBrowser::ProcessKey(eKeys Key) {
          //checking if page changed
          if ( pageFound && ttSetup.autoUpdatePage && cursorPos==0 && !selectingChannel && (PageCheckSum() != checkSum) ) {
             ShowPage();
+         //check if page was previously not found and is available now
+         } else if (!pageFound && CheckFirstSubPage(0)) {
+            ShowPage();
          } else {
             if (needClearMessage) {
                needClearMessage=false;
@@ -174,6 +182,9 @@ eOSState TeletextBrowser::ProcessKey(eKeys Key) {
             //updating clock
             UpdateClock();
          }
+         //check for activity timeout
+         if (ttSetup.inactivityTimeout && (time(NULL) - lastActivity > ttSetup.inactivityTimeout*60))
+            return osEnd;
          break;
       case kUp:
          if (selectingChannel) {
@@ -185,10 +196,10 @@ eOSState TeletextBrowser::ProcessKey(eKeys Key) {
             //fully revert cursor
             SetNumber(-3);
          }
-         ChangePageRelative();
+         ChangePageRelative(DirectionForward);
          Display::ShowUpperHalf();
          ShowPage();
-         break;        
+         break;
       case kDown:
          if (selectingChannel) {
              selectingChannel=false;
@@ -198,7 +209,7 @@ eOSState TeletextBrowser::ProcessKey(eKeys Key) {
             //fully reset
             SetNumber(-3);
          }
-         ChangePageRelative(true);
+         ChangePageRelative(DirectionBackward);
          Display::ShowUpperHalf();
          ShowPage();
          break;       
@@ -211,7 +222,7 @@ eOSState TeletextBrowser::ProcessKey(eKeys Key) {
             //fully reset
             SetNumber(-3);
          }
-         ChangeSubPageRelative();
+         ChangeSubPageRelative(DirectionForward);
          Display::ShowUpperHalf();
          ShowPage();
          break;       
@@ -225,7 +236,7 @@ eOSState TeletextBrowser::ProcessKey(eKeys Key) {
             SetNumber(-1);
             break;
          }
-         ChangeSubPageRelative(true);
+         ChangeSubPageRelative(DirectionBackward);
          Display::ShowUpperHalf();
          ShowPage();
          break; 
@@ -404,7 +415,7 @@ void TeletextBrowser::SetNumber(int i) {
    pageFound=true; //so that "page ... not found" is not displayed, but e.g. 1**-00
    if (++cursorPos>2) {
       cursorPos=0;
-      FindFirstSubPage(0);
+      CheckFirstSubPage(0);
       Display::ShowUpperHalf();
       ShowPage();
    } else {
@@ -422,17 +433,21 @@ static inline bool onlyDecimalDigits(int x) {
 }
 
 //after 199 comes 1A0, but if these pages exist, they contain no useful data, so filter them out
-int TeletextBrowser::nextValidPageNumber(int start, bool direction) {
+int TeletextBrowser::nextValidPageNumber(int start, Direction direction) {
    do {
-      if (direction)
+      switch (direction) {
+      case DirectionForward:
          start++;
-      else
+         break;
+      case DirectionBackward:
          start--;
+         break;
+      }
    } while (!onlyDecimalDigits(start));
    return start;
 }
 
-void TeletextBrowser::ChangePageRelative(bool back)
+void TeletextBrowser::ChangePageRelative(Direction direction)
 {
    int oldpage = currentPage;
    int oldSubPage = currentSubPage;
@@ -442,12 +457,11 @@ void TeletextBrowser::ChangePageRelative(bool back)
          currentPage--;
       else
          currentPage++;*/
-      currentPage=nextValidPageNumber(currentPage, !back);
+      currentPage=nextValidPageNumber(currentPage, direction);
       if (currentPage>0x899) currentPage=0x100;
       if (currentPage<0x100) currentPage=0x899;
       // sub page is always 0 if you change the page
-      FindFirstSubPage(0);
-      if (CheckPage()) {
+      if (CheckFirstSubPage(0)) {
          SetPreviousPage(oldpage, oldSubPage, currentPage);
          return;
       }
@@ -456,7 +470,7 @@ void TeletextBrowser::ChangePageRelative(bool back)
    return;
 }
 
-void TeletextBrowser::ChangeSubPageRelative(bool back)
+void TeletextBrowser::ChangeSubPageRelative(Direction direction)
 {
    int oldsubpage = currentSubPage;
 
@@ -465,7 +479,7 @@ void TeletextBrowser::ChangeSubPageRelative(bool back)
          currentSubPage--;
       else
          currentSubPage++;*/
-      currentSubPage=nextValidPageNumber(currentSubPage, !back);
+      currentSubPage=nextValidPageNumber(currentSubPage, direction);
       if (currentSubPage > 0x99) currentSubPage=0;
       if (currentSubPage < 0) currentSubPage=0x99;
 
@@ -476,20 +490,21 @@ void TeletextBrowser::ChangeSubPageRelative(bool back)
    return;
 }
 
-void TeletextBrowser::FindFirstSubPage(int startWith) {
+bool TeletextBrowser::CheckFirstSubPage(int startWith) {
    int oldsubpage = currentSubPage;
 
    do  {
       if (CheckPage())
-         return;
+         return true;
       //currentSubPage++;
-      currentSubPage=nextValidPageNumber(currentSubPage);
+      currentSubPage=nextValidPageNumber(currentSubPage, DirectionForward);
       
       if (currentSubPage > 0x99) currentSubPage=0;
       if (currentSubPage < 0) currentSubPage=0x99;
 
    } while (currentSubPage != oldsubpage);
    
+   return false;
 }
 
 bool TeletextBrowser::CheckPage()
@@ -568,7 +583,7 @@ bool TeletextBrowser::DecodePage() {
       }
    }
    
-   if ( (fd=s->openForReading(PageID(channel, currentPage, currentSubPage), true)) ) 
+   if ( (fd=s->openForReading(PageID(channel, currentPage, currentSubPage), true)) )
    {
       s->read(cache,sizeof cache,fd); // Read full page data
       s->close(fd);
@@ -605,7 +620,7 @@ int TeletextBrowser::PageCheckSum() {
    int retSum=0;
    StorageHandle fd;
    
-   FindFirstSubPage(currentSubPage);
+   CheckFirstSubPage(currentSubPage);
    
    Storage *s=Storage::instance();
    if ((fd=s->openForReading(PageID(channel, currentPage, currentSubPage), false)) ) {
@@ -639,9 +654,11 @@ void ChannelStatus::ChannelSwitch(const cDevice *Device, int ChannelNumber) {
 TeletextSetup ttSetup;
 
 TeletextSetup::TeletextSetup() {
+   //Set default values for setup options
+   
    configuredClrBackground=clrGray50;
    
-   //init key bindings with default values
+   //init key bindings
    for (int i=0;i<10;i++)
       mapKeyToAction[0]=(eTeletextAction)0;
    mapKeyToAction[3]=Zoom;
@@ -654,5 +671,11 @@ TeletextSetup::TeletextSetup() {
    //OSDHeight+width default values given in Start()
    OSDHAlign=50;
    OSDVAlign=50;
+   
+   //use the value set for VDR's min user inactivity.
+   //Initially this value could be changed via the plugin's setup, but I removed that
+   //because there is no advantage, but a possible problem when VDR's value is change
+   //after the plugin has stored its own value.
+   inactivityTimeout=Setup.MinUserInactivity;
 }
 
