@@ -110,6 +110,9 @@ Storage::Storage() {
    failedFreeSpace=false;
 }
 
+Storage::~Storage() {
+}
+
 void Storage::setSystem(StorageSystem s) {
    system=s;
 }
@@ -213,6 +216,8 @@ LegacyStorage::LegacyStorage() {
    pageBytes=TELETEXT_PAGESIZE;
 }
 
+LegacyStorage::~LegacyStorage() {
+}
 
 /*
 static inline int FilesForMegabytes(double MB, int blocksize) {
@@ -350,7 +355,7 @@ bool PackedStorage::seekTo(PageID page, int desc, bool create) {
    lseek(desc, 0, SEEK_SET);
    PageAddress addr[TOC_SIZE];
    
-   while (::read(desc, addr, sizeof(addr)) == sizeof(addr)) { 
+   while (::read(desc, addr, sizeof(addr)) == sizeof(addr)) {
       lseek(desc, 0, SEEK_CUR);
       for (int index=0; index<TOC_SIZE; index++) {
          if (addr[index]==page) {
@@ -363,7 +368,8 @@ bool PackedStorage::seekTo(PageID page, int desc, bool create) {
                lseek(desc, -(sizeof(addr)), SEEK_CUR);
                //update index
                addr[index]=page;
-               ::write(desc , addr, sizeof(addr));
+               if (::write(desc, addr, sizeof(addr)) != sizeof(addr))
+                  return false;
                //seek to data position
                lseek(desc, TELETEXT_PAGESIZE*index, SEEK_CUR);
                return true;
@@ -382,12 +388,14 @@ bool PackedStorage::seekTo(PageID page, int desc, bool create) {
       memset(addr, 0, sizeof(addr));
       //first entry is our page
       addr[0]=page;
-      ::write(desc, addr, sizeof(addr));
+      if (::write(desc, addr, sizeof(addr)) != sizeof(addr))
+         return false;
       //seek beyond end of file
       lseek(desc, (TELETEXT_PAGESIZE*TOC_SIZE)-1, SEEK_CUR);
       //write one byte to enlarge the file to the sought position
       char c=1;
-      ::write(desc, &c, 1);
+      if (::write(desc, &c, 1) != 1)
+         return false;
       //Now, calculate new file size
       byteCount += ( actualFileSize(lseek(desc, 0, SEEK_CUR)) - oldSize );
       //seek to beginning of data, which is requested
